@@ -36,12 +36,20 @@ void GmailSettings::save()
     mImapServer = "imaps://imap.gmail.com:993";
     mImapUsername = mEmailAddress;
 
-    mSmtpServer = "smtps://smtp.gmail.com:465";
+    mSmtpServer = "smtps://smtp.gmail.com:587";
     mSmtpUsername = mEmailAddress;
 
+    mCardDavServer = QStringLiteral("https://www.googleapis.com/carddav/v1/principals/") + mEmailAddress + "/";
+    mCardDavUsername = mEmailAddress;
+
+    mCalDavServer = QStringLiteral("https://www.google.com/calendar/dav/") + mEmailAddress + "/";
+    mCalDavUsername = mEmailAddress;
+
     saveAccount();
     saveImapResource();
     saveMailtransportResource();
+    saveCardDavResource();
+    saveCalDavResource();
     saveIdentity();
 }
 
