@@ -25,6 +25,7 @@
 #include <QFile>
 
 #include <sink/store.h>
+#include <sink/log.h>
 
 AccountFactory::AccountFactory(QObject *parent)
     : QObject(parent)
@@ -39,6 +40,8 @@ void AccountFactory::setAccountId(const QString &accountId)
             .then([this](const Sink::ApplicationDomain::SinkAccount &account) {
                 mAccountType = account.getProperty("type").toByteArray();
                 loadPackage();
+            }).onError([=](const KAsync::Error &error) {
+                SinkError() << "Failed to load the account: " << accountId << error;
             }).exec();
     }
 }
@@ -64,17 +67,20 @@ void AccountFactory::loadPackage()
     }();
     mUiPath.clear();
     mLoginUi.clear();
+    mAccountName.clear();
     mRequiresKeyring = false;
     if (pluginPath.isEmpty()) {
         qWarning() << "Failed to load account package: " << "org.kube.accounts." + mAccountType;
     } else {
         mUiPath = QUrl::fromLocalFile(pluginPath + "/AccountSettings.qml");
         mLoginUi = QUrl::fromLocalFile(pluginPath + "/Login.qml");
+        mAccountName = mAccountType;
         if (QFileInfo::exists(pluginPath + "/metadata.json")) {
             QFile file{pluginPath + "/metadata.json"};
             file.open(QIODevice::ReadOnly);
             auto json = QJsonDocument::fromJson(file.readAll());
             mRequiresKeyring = json.object().value("RequiresKeyring").toBool(true);
+            mAccountName = json.object().value("Name").toString();
         } else {
             mRequiresKeyring = true;
         }
