@@ -39,14 +39,12 @@ AccountsModel::~AccountsModel()
 
 QHash< int, QByteArray > AccountsModel::roleNames() const
 {
-    QHash<int, QByteArray> roles;
-
-    roles[Name] = "name";
-    roles[Icon] = "icon";
-    roles[AccountId] = "accountId";
-    roles[Status] = "status";
-
-    return roles;
+    return {
+        {Name, "name"},
+        {Icon, "icon"},
+        {AccountId, "accountId"},
+        {Status, "status"}
+    };
 }
 
 QVariant AccountsModel::data(const QModelIndex &idx, int role) const
@@ -84,7 +82,6 @@ void AccountsModel::runQuery(const Sink::Query &query)
 
 void AccountsModel::setAccountId(const QByteArray &accountId)
 {
-    qWarning() << "Setting account id" << accountId;
     if (accountId.isEmpty()) {
         setSourceModel(nullptr);
         mModel.clear();
@@ -106,7 +103,6 @@ QByteArray AccountsModel::accountId() const
 
 void AccountsModel::setResourceId(const QByteArray &resourceId)
 {
-    qWarning() << "Setting resource id" << resourceId;
     if (resourceId.isEmpty()) {
         setSourceModel(nullptr);
         mModel.clear();
