@@ -27,6 +27,7 @@
 #include <QUrlQuery>
 #include <QFileInfo>
 #include <QFile>
+#include <QTemporaryFile>
 #include <sink/store.h>
 #include <sink/log.h>
 
@@ -35,6 +36,7 @@
 #include "mime/mailtemplates.h"
 #include "mime/mailcrypto.h"
 #include "async.h"
+#include "sinkutils.h"
 
 std::vector<Crypto::Key> &operator+=(std::vector<Crypto::Key> &list, const std::vector<Crypto::Key> &add)
 {
@@ -43,6 +45,9 @@ std::vector<Crypto::Key> &operator+=(std::vector<Crypto::Key> &list, const std::
 }
 
 class IdentitySelector : public Selector {
+    Q_OBJECT
+    Q_PROPERTY (QString currentAccountId WRITE setCurrentAccountId)
+
 public:
     IdentitySelector(ComposerController &controller) : Selector(new IdentitiesModel), mController(controller)
     {
@@ -68,6 +73,16 @@ public:
 
     }
 
+    void setCurrentAccountId(const QString &accountId)
+    {
+        for (int i = 0; i < model()->rowCount(); i++) {
+            if (model()->index(i, 0).data(IdentitiesModel::AccountId).toString() == accountId) {
+                setCurrentIndex(i);
+                return;
+            }
+        }
+    }
+
     QVector<QByteArray> getAllAddresses()
     {
         QVector<QByteArray> list;
@@ -101,11 +116,11 @@ public:
     bool mFoundAllKeys = true;
 
     QSet<QByteArray> mMissingKeys;
-    AddresseeController() : Kube::ListPropertyController{{"name", "keyFound", "key"}}
+    AddresseeController() : Kube::ListPropertyController{{"name", "keyFound", "key", "fetching"}}
     {
         QObject::connect(
             this, &Kube::ListPropertyController::added, this, [this](const QByteArray &id, const QVariantMap &map) {
-                findKey(id, map.value("name").toString());
+                findKey(id, map.value("name").toString(), false);
             });
 
         QObject::connect(this, &Kube::ListPropertyController::removed, this, [this] (const QByteArray &id) {
@@ -125,19 +140,23 @@ public:
         emit foundAllKeysChanged();
     }
 
-    void findKey(const QByteArray &id, const QString &addressee)
+    void findKey(const QByteArray &id, const QString &addressee, bool fetchRemote)
     {
         mMissingKeys << id;
-        setFoundAllKeys(mMissingKeys.isEmpty());
+        setFoundAllKeys(false);
         KMime::Types::Mailbox mb;
         mb.fromUnicodeString(addressee);
 
         SinkLog() << "Searching key for: " << mb.address();
+
+        setValue(id, "fetching", fetchRemote);
+
         asyncRun<std::vector<Crypto::Key>>(this,
-            [mb] {
-                return Crypto::findKeys(QStringList{} << mb.address(), false, false);
+            [=] {
+                return Crypto::findKeys({mb.address()}, false, fetchRemote);
             },
             [this, addressee, id](const std::vector<Crypto::Key> &keys) {
+                setValue(id, "fetching", false);
                 if (!keys.empty()) {
                     if (keys.size() > 1) {
                         SinkWarning() << "Found more than one key, encrypting to all of them.";
@@ -153,6 +172,11 @@ public:
             });
     }
 
+    Q_INVOKABLE void fetchKeys(const QByteArray &id, const QString &addressee)
+    {
+        findKey(id, addressee, true);
+    }
+
     void set(const QStringList &list)
     {
         for (const auto &email : list) {
@@ -228,7 +252,7 @@ void ComposerController::findPersonalKey()
     auto identity = getIdentity();
     SinkLog() << "Looking for personal key for: " << identity.address();
     asyncRun<std::vector<Crypto::Key>>(this, [=] {
-            return Crypto::findKeys(QStringList{} << identity.address(), true);
+            return Crypto::findKeys({identity.address()}, true);
         },
         [this](const std::vector<Crypto::Key> &keys) {
             if (keys.empty()) {
@@ -280,16 +304,12 @@ static void applyAddresses(const QStringList &list, std::function<void(const QBy
     applyAddresses(mailboxes, callback);
 }
 
-static QStringList getStringListFromAddresses(const KMime::Types::Mailbox::List &s)
+static QStringList getStringListFromAddresses(const KMime::Types::Mailbox::List &mailboxes)
 {
     QStringList list;
-    applyAddresses(s, [&](const QByteArray &addrSpec, const QByteArray &displayName) {
-        if (displayName.isEmpty()) {
-            list << QString{addrSpec};
-        } else {
-            list << QString("%1 <%2>").arg(QString{displayName}).arg(QString{addrSpec});
-        }
-    });
+    for (const auto &mb : mailboxes) {
+        list << mb.prettyAddress(KMime::Types::Mailbox::QuoteWhenNecessary);
+    }
     return list;
 }
 
@@ -357,36 +377,44 @@ void ComposerController::setMessage(const KMime::Message::Ptr &msg)
 }
 
 void ComposerController::loadDraft(const QVariant &message) {
+    clear();
     loadMessage(message, [this] (const KMime::Message::Ptr &mail) {
-                mRemoveDraft = true;
-                setMessage(mail);
-            });
+        setEncrypt(KMime::isEncrypted(mail.data()));
+        setSign(KMime::isSigned(mail.data()));
+        mRemoveDraft = true;
+        setMessage(mail);
+    });
 }
 
 void ComposerController::loadReply(const QVariant &message) {
+    clear();
     loadMessage(message, [this] (const KMime::Message::Ptr &mail) {
-                //Find all personal email addresses to exclude from reply
-                KMime::Types::AddrSpecList me;
-                auto list = static_cast<IdentitySelector*>(mIdentitySelector.data())->getAllAddresses();
-                for (const auto &a : list) {
-                    KMime::Types::Mailbox mb;
-                    mb.setAddress(a);
-                    me << mb.addrSpec();
-                }
+        //Find all personal email addresses to exclude from reply
+        KMime::Types::AddrSpecList me;
+        auto list = static_cast<IdentitySelector*>(mIdentitySelector.data())->getAllAddresses();
+        for (const auto &a : list) {
+            KMime::Types::Mailbox mb;
+            mb.setAddress(a);
+            me << mb.addrSpec();
+        }
 
-                MailTemplates::reply(mail, [this] (const KMime::Message::Ptr &reply) {
-                            //We assume reply
-                            setMessage(reply);
-                        }, me);
-            });
+        setEncrypt(KMime::isEncrypted(mail.data()));
+        setSign(KMime::isSigned(mail.data()));
+        MailTemplates::reply(mail, [this] (const auto &msg) {
+            setMessage(msg);
+        }, me);
+    });
 }
 
 void ComposerController::loadForward(const QVariant &message) {
+    clear();
     loadMessage(message, [this] (const KMime::Message::Ptr &mail) {
-                MailTemplates::forward(mail, [this] (const KMime::Message::Ptr &fwdMessage) {
-                            setMessage(fwdMessage);
-                        });
-            });
+        setEncrypt(KMime::isEncrypted(mail.data()));
+        setSign(KMime::isSigned(mail.data()));
+        MailTemplates::forward(mail, [this] (const auto &msg) {
+            setMessage(msg);
+        });
+    });
 }
 
 void ComposerController::loadMessage(const QVariant &message, std::function<void(const KMime::Message::Ptr&)> callback)
@@ -398,8 +426,11 @@ void ComposerController::loadMessage(const QVariant &message, std::function<void
     Q_ASSERT(msg);
     Query query(*msg);
     query.request<Mail::MimeMessage>();
+    query.request<Mail::Draft>();
+    setLoading(true);
     Store::fetchOne<Mail>(query).then([this, callback](const Mail &mail) {
         setExistingMail(mail);
+        setLoading(false);
 
         const auto mailData = KMime::CRLFtoLF(mail.getMimeMessage());
         if (!mailData.isEmpty()) {
@@ -494,39 +525,23 @@ void ComposerController::send()
     }
 
     auto accountId = getAccountId();
-    //SinkLog() << "Sending a mail: " << *this;
-    using namespace Sink;
-    using namespace Sink::ApplicationDomain;
-
     Q_ASSERT(!accountId.isEmpty());
-    Query query;
-    query.containsFilter<SinkResource::Capabilities>(ResourceCapabilities::Mail::transport);
-    query.filter<SinkResource::Account>(accountId);
-    auto job = Store::fetchAll<SinkResource>(query)
-        .then([=](const QList<SinkResource::Ptr> &resources) {
-            if (!resources.isEmpty()) {
-                auto resourceId = resources[0]->identifier();
-                SinkLog() << "Sending message via resource: " << resourceId;
-                Mail mail(resourceId);
-                mail.setMimeMessage(message->encodedContent(true));
-                return Store::create(mail)
-                    .then<void>([=] {
-                        //Trigger a sync, but don't wait for it.
-                        Store::synchronize(Sink::SyncScope{}.resourceFilter(resourceId)).exec();
-                        if (mRemoveDraft) {
-                            SinkLog() << "Removing draft message.";
-                            //Remove draft
-                            Store::remove(getExistingMail()).exec();
-                        }
-                    });
+    if (accountId.isEmpty()) {
+        SinkWarning() << "No account id.";
+        return;
+    }
+    //SinkLog() << "Sending a mail: " << *this;
+    auto job = SinkUtils::sendMail(message->encodedContent(true), accountId.toUtf8())
+        .then([&] (const KAsync::Error &error) {
+            if (!error) {
+                if (mRemoveDraft) {
+                    SinkLog() << "Removing draft message.";
+                    Sink::Store::remove(getExistingMail()).exec();
+                }
             }
-            SinkWarning() << "Failed to find a mailtransport resource";
-            return KAsync::error<void>(0, "Failed to find a MailTransport resource.");
-        })
-        .then([&] (const KAsync::Error &) {
-            SinkLog() << "Message was sent: ";
             emit done();
         });
+
     run(job);
 }
 
@@ -534,6 +549,11 @@ void ComposerController::saveAsDraft()
 {
     SinkLog() << "Save as draft";
     const auto accountId = getAccountId();
+    Q_ASSERT(!accountId.isEmpty());
+    if (accountId.isEmpty()) {
+        SinkWarning() << "No account id.";
+        return;
+    }
     auto existingMail = getExistingMail();
 
     auto message = assembleMessage();
@@ -546,11 +566,11 @@ void ComposerController::saveAsDraft()
     using namespace Sink::ApplicationDomain;
 
     auto job = [&] {
-        if (existingMail.identifier().isEmpty()) {
-            SinkLog() << "Creating a new draft" << existingMail.identifier();
+        if (existingMail.identifier().isEmpty() || !existingMail.getDraft()) {
+            SinkLog() << "Creating a new draft" << existingMail.identifier() << "in account" << accountId;
             Query query;
             query.containsFilter<SinkResource::Capabilities>(ResourceCapabilities::Mail::drafts);
-            query.filter<SinkResource::Account>(accountId);
+            query.filter<SinkResource::Account>(accountId.toLatin1());
             return Store::fetchOne<SinkResource>(query)
                 .then([=](const SinkResource &resource) {
                     Mail mail(resource.identifier());
