@@ -43,6 +43,19 @@ void ControllerAction::execute()
     emit triggered();
 }
 
+void Controller::classBegin()
+{
+}
+
+void Controller::componentComplete()
+{
+    init();
+}
+
+void Controller::init()
+{
+}
+
 void Controller::clear()
 {
     auto meta = metaObject();
@@ -151,6 +164,34 @@ void ListPropertyController::setValue(const QByteArray &id, const QString &key,
     setValues(id, {{key, value}});
 }
 
+QVariant ListPropertyController::value(const QByteArray &id, const QString &key)
+{
+    QVariant result;
+    const auto idRole = mRoles["id"];
+    ::traverse(mModel.data(), [&] (QStandardItem *item) {
+        if (item->data(idRole).toByteArray() == id) {
+            result = item->data(mRoles[key]);
+            return false;
+        }
+        return true;
+    });
+    return result;
+}
+
+QByteArray ListPropertyController::findByProperty(const QByteArray &key, const QVariant &value) const
+{
+    QByteArray id;
+    const auto idRole = mRoles["id"];
+    ::traverse(mModel.data(), [&] (QStandardItem *item) {
+        if (item->data(mRoles[key]) == value) {
+            id = item->data(idRole).toByteArray();
+            return false;
+        }
+        return true;
+    });
+    return id;
+}
+
 void ListPropertyController::setValues(const QByteArray &id, const QVariantMap &values)
 {
     const auto idRole = mRoles["id"];
