@@ -20,6 +20,9 @@
 #include "kube_export.h"
 #include <QObject>
 #include <QVariant>
+#include <QAbstractItemModel>
+#include <QStandardItemModel>
+#include <QQmlParserStatus>
 #include <KAsync/Async>
 
 #define KUBE_CONTROLLER_PROPERTY(TYPE, NAME, LOWERCASENAME) \
@@ -48,9 +51,6 @@
     public: Kube::ListPropertyController* NAME##Controller() const { Q_ASSERT(controller_##NAME); return controller_##NAME.data(); } \
 
 
-class QAbstractItemModel;
-class QStandardItemModel;
-
 namespace Kube {
 
 class ControllerState : public QObject {
@@ -88,12 +88,19 @@ signals:
     void triggered();
 };
 
-class Controller : public QObject {
+class Controller : public QObject, public QQmlParserStatus {
     Q_OBJECT
+    Q_INTERFACES(QQmlParserStatus)
+
 public:
     Controller() = default;
     virtual ~Controller() = default;
 
+    virtual void init();
+
+    void classBegin() override;
+    void componentComplete() override;
+
 public slots:
     virtual void clear();
 
@@ -106,7 +113,7 @@ protected:
     void run(const KAsync::Job<void> &job);
 };
 
-class ListPropertyController : public QObject
+class KUBE_EXPORT ListPropertyController : public QObject
 {
     Q_OBJECT
     Q_PROPERTY(QAbstractItemModel* model READ model CONSTANT)
@@ -122,8 +129,11 @@ public:
 
     void setValue(const QByteArray &id, const QString &key, const QVariant &);
     void setValues(const QByteArray &id, const QVariantMap &values);
+    QVariant value(const QByteArray &id, const QString &key);
     void traverse(const std::function<void(const QVariantMap &)> &f);
 
+    QByteArray findByProperty(const QByteArray &key, const QVariant &) const;
+
     template<typename T>
     QList<T> getList(const QString &property)
     {
