@@ -19,22 +19,16 @@
 
 #pragma once
 
-#include "framework/src/errors.h"
+#include <KMime/Content>
 
-#include <KMime/Message>
-
-#include <QByteArray>
-#include <QVariant>
-
-#include <functional>
 #include <memory>
-#include "crypto.h"
+
+#include <errors.h>
+#include <crypto.h>
 
 namespace MailCrypto {
 
-Expected<Crypto::Error, std::unique_ptr<KMime::Content>>
-processCrypto(std::unique_ptr<KMime::Content> content, const std::vector<Crypto::Key> &signingKeys,
-    const std::vector<Crypto::Key> &encryptionKeys, const Crypto::Key &attachedKey);
+Expected<Crypto::Error, std::unique_ptr<KMime::Content>> processCrypto(std::unique_ptr<KMime::Content> content, const std::vector<Crypto::Key> &signingKeys, const std::vector<Crypto::Key> &encryptionKeys);
 
-}; // namespace MailCrypto
+};
 
