@@ -575,12 +575,12 @@ static KMime::Types::Mailbox::List getMailingListAddresses(const KMime::Message:
     return mailingListAddresses;
 }
 
-struct Recipients {
+struct RecipientMailboxes {
     KMime::Types::Mailbox::List to;
     KMime::Types::Mailbox::List cc;
 };
 
-static Recipients getRecipients(const KMime::Message::Ptr &origMsg, const KMime::Types::AddrSpecList &me)
+static RecipientMailboxes getRecipients(const KMime::Message::Ptr &origMsg, const KMime::Types::AddrSpecList &me)
 {
     const KMime::Types::Mailbox::List replyToList = origMsg->replyTo()->mailboxes();
     const KMime::Types::Mailbox::List mailingListAddresses = getMailingListAddresses(origMsg);
@@ -807,7 +807,7 @@ void MailTemplates::reply(const KMime::Message::Ptr &origMsg, const std::functio
 
     MimeTreeParser::ObjectTreeParser otp;
     otp.parseObjectTree(origMsg.data());
-    otp.decryptParts();
+    otp.decryptAndVerify();
     const auto plainTextContent = otp.plainTextContent();
     const auto htmlContent = otp.htmlContent();
 
@@ -866,7 +866,7 @@ void MailTemplates::forward(const KMime::Message::Ptr &origMsg,
         qDebug() << "Original message was encrypted, decrypting it";
         MimeTreeParser::ObjectTreeParser otp;
         otp.parseObjectTree(origMsg.data());
-        otp.decryptParts();
+        otp.decryptAndVerify();
 
         auto htmlContent = otp.htmlContent();
 
@@ -946,7 +946,7 @@ QString MailTemplates::body(const KMime::Message::Ptr &msg, bool &isHtml)
     return html;
 }
 
-static KMime::Content *createAttachmentPart(const QByteArray &content, const QString &filename, bool isInline, const QByteArray &mimeType, const QString &name)
+static KMime::Content *createAttachmentPart(const QByteArray &content, const QString &filename, bool isInline, const QByteArray &mimeType, const QString &name, bool base64Encode = true)
 {
 
     KMime::Content *part = new KMime::Content;
@@ -958,10 +958,10 @@ static KMime::Content *createAttachmentPart(const QByteArray &content, const QSt
     }
 
     part->contentType(true)->setMimeType(mimeType);
-    part->contentType(true)->setName(name, "utf-8");
-    // Just always encode attachments base64 so it's safe for binary data,
-    // except when it's another message
-    if(mimeType != "message/rfc822") {
+    if (!name.isEmpty()) {
+        part->contentType(true)->setName(name, "utf-8");
+    }
+    if(base64Encode) {
         part->contentTransferEncoding(true)->setEncoding(KMime::Headers::CEbase64);
     }
     part->setBody(content);
@@ -981,11 +981,34 @@ static KMime::Types::Mailbox::List stringListToMailboxes(const QStringList &list
     for (const auto &s : list) {
         KMime::Types::Mailbox mb;
         mb.fromUnicodeString(s);
-        mailboxes << mb;
+        if (mb.hasAddress()) {
+            mailboxes << mb;
+        } else {
+            qWarning() << "Got an invalid address: " << s << list;
+            Q_ASSERT(false);
+        }
     }
     return mailboxes;
 }
 
+
+static void setRecipients(KMime::Message &message, const Recipients &recipients)
+{
+    message.to(true)->clear();
+    for (const auto &mb : stringListToMailboxes(recipients.to)) {
+        message.to()->addAddress(mb);
+    }
+    message.cc(true)->clear();
+    for (const auto &mb : stringListToMailboxes(recipients.cc)) {
+        message.cc()->addAddress(mb);
+    }
+    message.bcc(true)->clear();
+    for (const auto &mb : stringListToMailboxes(recipients.bcc)) {
+        message.bcc()->addAddress(mb);
+    }
+}
+
+
 KMime::Message::Ptr MailTemplates::createMessage(KMime::Message::Ptr existingMessage,
     const QStringList &to, const QStringList &cc, const QStringList &bcc,
     const KMime::Types::Mailbox &from, const QString &subject, const QString &body, bool htmlBody,
@@ -1004,18 +1027,7 @@ KMime::Message::Ptr MailTemplates::createMessage(KMime::Message::Ptr existingMes
     mail->date()->setDateTime(QDateTime::currentDateTime());
     mail->userAgent()->fromUnicodeString(QString("%1/%2(%3)").arg(QString::fromLocal8Bit("Kube")).arg("0.1").arg(QSysInfo::prettyProductName()), "utf-8");
 
-    mail->to(true)->clear();
-    for (const auto &mb : stringListToMailboxes(to)) {
-        mail->to()->addAddress(mb);
-    }
-    mail->cc(true)->clear();
-    for (const auto &mb : stringListToMailboxes(cc)) {
-        mail->cc()->addAddress(mb);
-    }
-    mail->bcc(true)->clear();
-    for (const auto &mb : stringListToMailboxes(bcc)) {
-        mail->bcc()->addAddress(mb);
-    }
+    setRecipients(*mail, {to, cc, bcc});
 
     mail->from(true)->clear();
     mail->from(true)->addAddress(from);
@@ -1031,16 +1043,42 @@ KMime::Message::Ptr MailTemplates::createMessage(KMime::Message::Ptr existingMes
     }
     mail->assemble();
 
+    const bool encryptionRequired = !signingKeys.empty() || !encryptionKeys.empty();
+    //We always attach the key when encryption is enabled.
+    const bool attachingPersonalKey = encryptionRequired;
+
+    auto allAttachments = attachments;
+    if (attachingPersonalKey) {
+        const auto publicKeyExportResult = Crypto::exportPublicKey(attachedKey);
+        if (!publicKeyExportResult) {
+            qWarning() << "Failed to export public key" << publicKeyExportResult.error();
+            return {};
+        }
+        const auto publicKeyData = publicKeyExportResult.value();
+        allAttachments << Attachment{
+            {},
+            QString("0x%1.asc").arg(QString{attachedKey.shortKeyId}),
+            "application/pgp-keys",
+            false,
+            publicKeyData
+        };
+    }
+
     std::unique_ptr<KMime::Content> bodyPart{[&] {
-        if (!attachments.isEmpty()) {
+        if (!allAttachments.isEmpty()) {
             auto bodyPart = new KMime::Content;
             bodyPart->contentType(true)->setMimeType("multipart/mixed");
             bodyPart->contentType()->setBoundary(KMime::multiPartBoundary());
             bodyPart->contentTransferEncoding()->setEncoding(KMime::Headers::CE7Bit);
             bodyPart->setPreamble("This is a multi-part message in MIME format.\n");
             bodyPart->addContent(createBodyPart(body, htmlBody));
-            for (const auto &attachment : attachments) {
-                bodyPart->addContent(createAttachmentPart(attachment.data, attachment.filename, attachment.isInline, attachment.mimeType, attachment.name));
+            for (const auto &attachment : allAttachments) {
+
+                // Just always encode attachments base64 so it's safe for binary data,
+                // except when it's another message or an ascii armored key
+                static QSet<QString> noEncodingRequired{{"message/rfc822"}, {"application/pgp-keys"}};
+                const bool base64Encode = !noEncodingRequired.contains(attachment.mimeType);
+                bodyPart->addContent(createAttachmentPart(attachment.data, attachment.filename, attachment.isInline, attachment.mimeType, attachment.name, base64Encode));
             }
             return bodyPart;
         } else {
@@ -1049,20 +1087,25 @@ KMime::Message::Ptr MailTemplates::createMessage(KMime::Message::Ptr existingMes
     }()};
     bodyPart->assemble();
 
-    QByteArray bodyData;
-    if (!signingKeys.empty() || !encryptionKeys.empty()) {
-        auto result = MailCrypto::processCrypto(std::move(bodyPart), signingKeys, encryptionKeys, attachedKey);
-        if (!result) {
-            qWarning() << "Crypto failed";
-            return {};
+    const QByteArray bodyData = [&] {
+        if (encryptionRequired) {
+            auto result = MailCrypto::processCrypto(std::move(bodyPart), signingKeys, encryptionKeys);
+            if (!result) {
+                qWarning() << "Crypto failed" << result.error();
+                return QByteArray{};
+            }
+            result.value()->assemble();
+            return result.value()->encodedContent();
+        } else {
+            if (!bodyPart->contentType(false)) {
+                bodyPart->contentType(true)->setMimeType("text/plain");
+                bodyPart->assemble();
+            }
+            return bodyPart->encodedContent();
         }
-        bodyData = result.value()->encodedContent();
-    } else {
-        if (!bodyPart->contentType(false)) {
-            bodyPart->contentType(true)->setMimeType("text/plain");
-            bodyPart->assemble();
-        }
-        bodyData = bodyPart->encodedContent();
+    }();
+    if (bodyData.isEmpty()) {
+        return {};
     }
 
     KMime::Message::Ptr resultMessage(new KMime::Message);
@@ -1070,3 +1113,49 @@ KMime::Message::Ptr MailTemplates::createMessage(KMime::Message::Ptr existingMes
     resultMessage->parse(); // Not strictly necessary.
     return resultMessage;
 }
+
+
+KMime::Message::Ptr MailTemplates::createIMipMessage(
+    const QString &from,
+    const Recipients &recipients,
+    const QString &subject,
+    const QString &body,
+    const QString &attachment)
+{
+    KMime::Message::Ptr message = KMime::Message::Ptr( new KMime::Message );
+    message->contentTransferEncoding()->clear();  // 7Bit, decoded.
+
+    // Set the headers
+    message->userAgent()->fromUnicodeString(QString("%1/%2(%3)").arg(QString::fromLocal8Bit("Kube")).arg("0.1").arg(QSysInfo::prettyProductName()), "utf-8");
+    message->from()->fromUnicodeString(from, "utf-8");
+
+    setRecipients(*message, recipients);
+
+    message->date()->setDateTime(QDateTime::currentDateTime());
+    message->subject()->fromUnicodeString(subject, "utf-8");
+    message->contentType()->setMimeType("multipart/alternative");
+    message->contentType()->setBoundary(KMime::multiPartBoundary());
+
+    // Set the first multipart, the body message.
+    KMime::Content *bodyMessage = new KMime::Content{message.data()};
+    bodyMessage->contentType()->setMimeType("text/plain");
+    bodyMessage->contentType()->setCharset("utf-8");
+    bodyMessage->contentTransferEncoding()->setEncoding(KMime::Headers::CEquPr);
+    bodyMessage->setBody(KMime::CRLFtoLF(body.toUtf8()));
+    message->addContent( bodyMessage );
+
+    // Set the second multipart, the attachment.
+    KMime::Content *attachMessage = new KMime::Content{message.data()};
+    attachMessage->contentDisposition()->setDisposition(KMime::Headers::CDattachment);
+    attachMessage->contentType()->setMimeType("text/calendar");
+    attachMessage->contentType()->setCharset("utf-8");
+    attachMessage->contentType()->setName(QLatin1String("event.ics"), "utf-8");
+    attachMessage->contentType()->setParameter(QLatin1String("method"), QLatin1String("REPLY"));
+    attachMessage->contentTransferEncoding()->setEncoding(KMime::Headers::CEquPr);
+    attachMessage->setBody(KMime::CRLFtoLF(attachment.toUtf8()));
+    message->addContent(attachMessage);
+
+    // Job done, attach the both multiparts and assemble the message.
+    message->assemble();
+    return message;
+}
