@@ -558,35 +558,43 @@ QString MimeMessagePart::htmlContent() const
 AlternativeMessagePart::AlternativeMessagePart(ObjectTreeParser *otp, KMime::Content *node)
     : MessagePart(otp, QString(), node)
 {
-    if (auto dataIcal = findTypeInDirectChilds(mNode, "text/calendar")) {
+    if (auto dataIcal = findTypeInDirectChildren(mNode, "text/calendar")) {
         mChildParts[Util::MultipartIcal] = MimeMessagePart::Ptr(new MimeMessagePart(mOtp, dataIcal, true));
     }
 
-    if (auto dataText = findTypeInDirectChilds(mNode, "text/plain")) {
+    if (auto dataText = findTypeInDirectChildren(mNode, "text/plain")) {
         mChildParts[Util::MultipartPlain] = MimeMessagePart::Ptr(new MimeMessagePart(mOtp, dataText, true));
     }
 
-    if (auto dataHtml = findTypeInDirectChilds(mNode, "text/html")) {
+    if (auto dataHtml = findTypeInDirectChildren(mNode, "text/html")) {
         mChildParts[Util::MultipartHtml] = MimeMessagePart::Ptr(new MimeMessagePart(mOtp, dataHtml, true));
     } else {
         // If we didn't find the HTML part as the first child of the multipart/alternative, it might
         // be that this is a HTML message with images, and text/plain and multipart/related are the
         // immediate children of this multipart/alternative node.
         // In this case, the HTML node is a child of multipart/related.
-        if (auto data = findTypeInDirectChilds(mNode, "multipart/related")) {
+        // In the case of multipart/related we don't expect multiple html parts, it is usually used to group attachments
+        // with html content.
+        //
+        // In any case, this is not a complete implementation of MIME, but an approximation for the kind of mails we actually see in the wild.
+        auto data  = [&] {
+            if (auto d = findTypeInDirectChildren(mNode, "multipart/related")) {
+                return d;
+            }
+            return findTypeInDirectChildren(mNode, "multipart/mixed");
+        }();
+        if (data) {
+            QString htmlContent;
             const auto parts = data->contents();
-            for (int i = 0; i < parts.size(); i++) {
-                const auto p = parts.at(i);
-                if (i == 0 ) {
-                    mChildParts[Util::MultipartHtml] = MimeMessagePart::Ptr(new MimeMessagePart(mOtp, p, true));
+            for (auto p : data->contents()) {
+                if ((!p->contentType()->isEmpty())
+                    && (p->contentType()->mimeType() == "text/html")) {
+                    htmlContent += MimeMessagePart(mOtp, p, true).text();
                 } else if (KMime::isAttachment(p)) {
                     appendSubPart(MimeMessagePart::Ptr(new MimeMessagePart(otp, p, true)));
                 }
             }
-        //Same for multipart/mixed
-        } else if (auto data  = findTypeInDirectChilds(mNode, "multipart/mixed")) {
-            //FIXME: This doesn't populate mChildParts but instead attaches subparts. That way we at least process all parts as we should.
-            parseInternal(data);
+            mChildParts[Util::MultipartHtml] = MessagePart::Ptr(new MessagePart(mOtp, htmlContent, nullptr));
         }
     }
 }
@@ -628,6 +636,14 @@ QString AlternativeMessagePart::htmlContent() const
     }
 }
 
+QString AlternativeMessagePart::icalContent() const
+{
+    if (mChildParts.contains(Util::MultipartIcal)) {
+        return mChildParts[Util::MultipartIcal]->text();
+    }
+    return {};
+}
+
 //-----CertMessageBlock----------------------
 
 CertMessagePart::CertMessagePart(ObjectTreeParser *otp, KMime::Content *node, const CryptoProtocol cryptoProto)
@@ -647,8 +663,7 @@ CertMessagePart::~CertMessagePart()
 
 void CertMessagePart::import()
 {
-    const QByteArray certData = mNode->decodedContent();
-    importKeys(mProtocol, certData);
+    importKey(mProtocol, mNode->decodedContent());
 }
 
 QString CertMessagePart::text() const
@@ -696,7 +711,14 @@ static QString prettifyDN(const char *uid)
 
 void SignedMessagePart::sigStatusToMetaData(const Signature &signature)
 {
-    mMetaData.isGoodSignature = signature.status == GPG_ERR_NO_ERROR;
+    mMetaData.isGoodSignature = signature.status.errorCode() == GPG_ERR_NO_ERROR;
+    if (!mMetaData.isGoodSignature) {
+        if (signature.status.errorCode() == GPG_ERR_NO_PUBKEY) {
+            qWarning() << "No public key to verify signature.";
+        } else {
+            qWarning() << "Is no good signature" << signature.status;
+        }
+    }
     // save extended signature status flags
     auto summary = signature.summary;
     mMetaData.keyMissing = summary & GPGME_SIGSUM_KEY_MISSING;
@@ -927,8 +949,15 @@ bool EncryptedMessagePart::okDecryptMIME(KMime::Content &data)
         return true;
     }
 
-    if (mMetaData.isEncrypted && decryptResult.recipients.size()) {
-        mMetaData.keyId = decryptResult.recipients.at(0).keyId;
+    if (mMetaData.isEncrypted) {
+        mMetaData.keyId = [&] {
+            foreach (const auto &recipient, decryptResult.recipients) {
+                if (recipient.status.errorCode() != GPG_ERR_NO_SECKEY) {
+                    return recipient.keyId;
+                }
+            }
+            return QByteArray{};
+        }();
     }
 
     if (!decryptResult.error) {
@@ -939,14 +968,7 @@ bool EncryptedMessagePart::okDecryptMIME(KMime::Content &data)
         mMetaData.isEncrypted = errorCode != GPG_ERR_NO_DATA;
         qWarning() << "Failed to decrypt : " << decryptResult.error;
 
-        const bool noSecretKeyAvilable = [&] {
-            foreach (const auto &recipient, decryptResult.recipients) {
-                if (!(recipient.status.errorCode() == GPG_ERR_NO_SECKEY)) {
-                    return false;
-                }
-            }
-            return true;
-        }();
+        const bool noSecretKeyAvilable = mMetaData.keyId.isEmpty();
         bool passphraseError = errorCode == GPG_ERR_CANCELED || errorCode == GPG_ERR_NO_SECKEY;
         //We only get a decryption failed error when we enter the wrong passphrase....
         if (!passphraseError && !noSecretKeyAvilable) {
@@ -955,13 +977,13 @@ bool EncryptedMessagePart::okDecryptMIME(KMime::Content &data)
 
         if(noSecretKeyAvilable) {
             mError = NoKeyError;
-            mMetaData.errorText = tr("Could not decrypt the data. ") + tr("No key found for recepients.");
+            mMetaData.errorText = tr("Could not decrypt the data: no key found for recipients.");
         } else if (passphraseError) {
             mError = PassphraseError;
             // mMetaData.errorText = QString::fromLocal8Bit(decryptResult.error().asString());
         } else {
             mError = UnknownError;
-            mMetaData.errorText = tr("Could not decrypt the data. ");
+            mMetaData.errorText = tr("Could not decrypt the data.");
             //                         + tr("Error: %1").arg(QString::fromLocal8Bit(decryptResult.error().asString()));
         }
         return false;
