@@ -242,10 +242,11 @@ public:
 
     QString plaintextContent() const Q_DECL_OVERRIDE;
     QString htmlContent() const Q_DECL_OVERRIDE;
+    QString icalContent() const;
 
     QList<Util::HtmlMode> availableModes();
 private:
-    QMap<Util::HtmlMode, MimeMessagePart::Ptr> mChildParts;
+    QMap<Util::HtmlMode, MessagePart::Ptr> mChildParts;
 
     friend class DefaultRendererPrivate;
     friend class ObjectTreeParser;
@@ -317,7 +318,7 @@ public:
 
 private:
     /** Handles the dectyptioon of a given content
-     * returns true if the decryption was successfull
+     * returns true if the decryption was successful
      * if used in async mode, check if mMetaData.inProgress is true, it inicates a running decryption process.
      */
     bool okDecryptMIME(KMime::Content &data);
