@@ -48,11 +48,9 @@
 #include "bodypartformatter.h"
 #include "util.h"
 
-#include <KMime/Headers>
 #include <KMime/Message>
 
 #include <QByteArray>
-#include <QTextCodec>
 #include <QUrl>
 #include <QMimeDatabase>
 #include <QTextStream>
@@ -271,6 +269,7 @@ QVector<MessagePart::Ptr> ObjectTreeParser::collectContentParts(MessagePart::Ptr
             } else if (dynamic_cast<MimeTreeParser::AlternativeMessagePart*>(part.data())) {
                 return true;
             } else if (dynamic_cast<MimeTreeParser::HtmlMessagePart*>(part.data())) {
+                //Don't if we have an alternative part as parent
                 return true;
             } else if (dynamic_cast<MimeTreeParser::EncapsulatedRfc822MessagePart*>(part.data())) {
                 if (start.data() == part.data()) {
@@ -306,6 +305,11 @@ QVector<MessagePart::Ptr> ObjectTreeParser::collectAttachmentParts()
 }
 
 void ObjectTreeParser::decryptParts()
+{
+    decryptAndVerify();
+}
+
+void ObjectTreeParser::decrypt()
 {
     ::collect(mParsedPart,
         [] (const MessagePartPtr &) { return true; },
@@ -315,6 +319,10 @@ void ObjectTreeParser::decryptParts()
             }
             return false;
         });
+}
+
+void ObjectTreeParser::verifySignatures()
+{
     ::collect(mParsedPart,
         [] (const MessagePartPtr &) { return true; },
         [] (const MessagePartPtr &part) {
@@ -323,6 +331,13 @@ void ObjectTreeParser::decryptParts()
             }
             return false;
         });
+
+}
+
+void ObjectTreeParser::decryptAndVerify()
+{
+    decrypt();
+    verifySignatures();
 }
 
 void ObjectTreeParser::importCertificates()
@@ -399,6 +414,11 @@ MessagePartPtr ObjectTreeParser::parsedPart() const
     return mParsedPart;
 }
 
+/*
+ * This will lookup suitable formatters based on the type,
+ * and let them generate a list of parts.
+ * If the formatter generated a list of parts, then those are taken, otherwise we move on to the next match.
+ */
 QVector<MessagePartPtr> ObjectTreeParser::processType(KMime::Content *node, const QByteArray &mediaType, const QByteArray &subType)
 {
     static MimeTreeParser::BodyPartFormatterBaseFactory factory;
