@@ -29,7 +29,7 @@ QByteArray readMailFromFile(const QString &mailFile)
     return file.readAll();
 }
 
-class InterfaceTest : public QObject
+class MimeTreeParserTest : public QObject
 {
     Q_OBJECT
 private slots:
@@ -419,7 +419,7 @@ private slots:
         QCOMPARE(otp.plainTextContent(), QString::fromUtf8("test\n\n-- \nThis is a HTML signature.\n"));
 
         auto signaturePart = part->signatures().first();
-        QCOMPARE(signaturePart->partMetaData()->keyId, QByteArray{"2E3B7787B1B75920"});
+        QVERIFY(signaturePart->partMetaData()->keyId.endsWith(QByteArray{"2E3B7787B1B75920"}));
         //We lack the public key for this message
         QCOMPARE(signaturePart->partMetaData()->isGoodSignature, false);
         QCOMPARE(signaturePart->partMetaData()->keyMissing, true);
@@ -432,9 +432,11 @@ private slots:
         otp.decryptParts();
         auto partList = otp.collectContentParts();
         QCOMPARE(partList.size(), 1);
-        auto part = partList[0].dynamicCast<MimeTreeParser::MessagePart>();
+        auto part = partList[0].dynamicCast<MimeTreeParser::AlternativeMessagePart>();
+        QVERIFY(part);
         QCOMPARE(part->encryptions().size(), 0);
         QCOMPARE(part->signatures().size(), 0);
+        QVERIFY(part->isHtml());
         QCOMPARE(otp.plainTextContent(), QString::fromUtf8("Hi,\n\nThis is an HTML message with attachments.\n\nCheers,\nChristian"));
         QCOMPARE(otp.htmlContent(), QString::fromUtf8("<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=us-ascii\"></head><body style=\"word-wrap: break-word; -webkit-nbsp-mode: space; line-break: after-white-space;\" class=\"\"><meta http-equiv=\"Content-Type\" content=\"text/html; charset=us-ascii\" class=\"\"><div style=\"word-wrap: break-word; -webkit-nbsp-mode: space; line-break: after-white-space;\" class=\"\">Hi,<div class=\"\"><br class=\"\"></div><blockquote style=\"margin: 0 0 0 40px; border: none; padding: 0px;\" class=\"\"><div class=\"\">This is an <b class=\"\">HTML</b> message with attachments.</div></blockquote><div class=\"\"><br class=\"\"></div><div class=\"\">Cheers,</div><div class=\"\">Christian<img apple-inline=\"yes\" id=\"B9EE68A9-83CA-41CD-A3E4-E5BA301F797A\" class=\"\" src=\"cid:F5B62D1D-E4EC-4C59-AA5A-708525C2AC3C\"></div></div></body></html>"));
 
@@ -449,17 +451,57 @@ private slots:
         otp.decryptParts();
         otp.print();
         auto partList = otp.collectContentParts();
-        QCOMPARE(partList.size(), 2);
-        auto part = partList[0].dynamicCast<MimeTreeParser::MessagePart>();
+        QCOMPARE(partList.size(), 1);
+        auto part = partList[0].dynamicCast<MimeTreeParser::AlternativeMessagePart>();
+        QVERIFY(part);
         QCOMPARE(part->encryptions().size(), 0);
         QCOMPARE(part->signatures().size(), 0);
+        QVERIFY(part->isHtml());
         QCOMPARE(otp.plainTextContent(), QString::fromUtf8("Hello\n\n\n\nRegards\n\nFsdfsdf"));
         QCOMPARE(otp.htmlContent(), QString::fromUtf8("<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=us-ascii\"></head><body style=\"word-wrap: break-word; -webkit-nbsp-mode: space; line-break: after-white-space;\" class=\"\"><strike class=\"\">Hello</strike><div class=\"\"><br class=\"\"></div><div class=\"\"></div></body></html><html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=us-ascii\"></head><body style=\"word-wrap: break-word; -webkit-nbsp-mode: space; line-break: after-white-space;\" class=\"\"><div class=\"\"></div><div class=\"\"><br class=\"\"></div><div class=\"\"><b class=\"\">Regards</b></div><div class=\"\"><b class=\"\"><br class=\"\"></b></div><div class=\"\">Fsdfsdf</div></body></html>"));
 
         auto attachments = otp.collectAttachmentParts();
         QCOMPARE(attachments.size(), 1);
     }
+
+    void testInvitation()
+    {
+        MimeTreeParser::ObjectTreeParser otp;
+        otp.parseObjectTree(readMailFromFile("invitation.mbox"));
+        otp.decryptParts();
+        otp.print();
+        auto partList = otp.collectContentParts();
+        QCOMPARE(partList.size(), 1);
+        auto part = partList[0].dynamicCast<MimeTreeParser::AlternativeMessagePart>();
+        QVERIFY(part);
+        QCOMPARE(part->encryptions().size(), 0);
+        QCOMPARE(part->signatures().size(), 0);
+        QVERIFY(!part->isHtml());
+        QVERIFY(part->availableModes().contains(MimeTreeParser::Util::MultipartIcal));
+
+        auto attachments = otp.collectAttachmentParts();
+        QCOMPARE(attachments.size(), 0);
+    }
+
+    void testGmailInvitation()
+    {
+        MimeTreeParser::ObjectTreeParser otp;
+        otp.parseObjectTree(readMailFromFile("gmail-invitation.mbox"));
+        otp.decryptParts();
+        otp.print();
+        auto partList = otp.collectContentParts();
+        QCOMPARE(partList.size(), 1);
+        auto part = partList[0].dynamicCast<MimeTreeParser::AlternativeMessagePart>();
+        QVERIFY(part);
+        QCOMPARE(part->encryptions().size(), 0);
+        QCOMPARE(part->signatures().size(), 0);
+        QVERIFY(part->isHtml());
+        QVERIFY(part->availableModes().contains(MimeTreeParser::Util::MultipartIcal));
+
+        auto attachments = otp.collectAttachmentParts();
+        QCOMPARE(attachments.size(), 1);
+    }
 };
 
-QTEST_GUILESS_MAIN(InterfaceTest)
+QTEST_GUILESS_MAIN(MimeTreeParserTest)
 #include "mimetreeparsertest.moc"
