@@ -29,14 +29,35 @@ class PartModelPrivate
 {
 public:
     PartModelPrivate(PartModel *q_ptr, const std::shared_ptr<MimeTreeParser::ObjectTreeParser> &parser);
-    ~PartModelPrivate();
+    ~PartModelPrivate() = default;
+
+    void checkPart(const MimeTreeParser::MessagePart::Ptr part) {
+        //Has to contain html, and be an alternative part (so it's not only html)
+        if (part->isHtml() && part.dynamicCast<MimeTreeParser::AlternativeMessagePart>()) {
+            containsHtmlAndPlain = true;
+            emit q->containsHtmlChanged();
+        }
+    }
+
+    //Recursively find encapsulated messages
+    void findEncapsulated(const MimeTreeParser::EncapsulatedRfc822MessagePart::Ptr &e) {
+        mEncapsulatedParts[e.data()] = mParser->collectContentParts(e);
+        for (auto subPart : mEncapsulatedParts[e.data()]) {
+            checkPart(subPart);
+            mParents[subPart.data()] = e.data();
+            if (auto encapsulatedSub = subPart.dynamicCast<MimeTreeParser::EncapsulatedRfc822MessagePart>()) {
+                findEncapsulated(encapsulatedSub);
+            }
+        }
+    }
 
-    void findEncapsulated(const MimeTreeParser::EncapsulatedRfc822MessagePart::Ptr &e);
     PartModel *q;
     QVector<MimeTreeParser::MessagePartPtr> mParts;
     QHash<MimeTreeParser::MessagePart*, QVector<MimeTreeParser::MessagePartPtr>> mEncapsulatedParts;
     QHash<MimeTreeParser::MessagePart*, MimeTreeParser::MessagePart*> mParents;
     std::shared_ptr<MimeTreeParser::ObjectTreeParser> mParser;
+    bool showHtml{false};
+    bool containsHtmlAndPlain{false};
 };
 
 PartModelPrivate::PartModelPrivate(PartModel *q_ptr, const std::shared_ptr<MimeTreeParser::ObjectTreeParser> &parser)
@@ -45,28 +66,13 @@ PartModelPrivate::PartModelPrivate(PartModel *q_ptr, const std::shared_ptr<MimeT
 {
     mParts = mParser->collectContentParts();
     for (auto p : mParts) {
+        checkPart(p);
         if (auto e = p.dynamicCast<MimeTreeParser::EncapsulatedRfc822MessagePart>()) {
             findEncapsulated(e);
         }
     }
 }
 
-//Recursively find encapsulated messages
-void PartModelPrivate::findEncapsulated(const MimeTreeParser::EncapsulatedRfc822MessagePart::Ptr &e)
-{
-    mEncapsulatedParts[e.data()] = mParser->collectContentParts(e);
-    for (auto subPart : mEncapsulatedParts[e.data()]) {
-        mParents[subPart.data()] = e.data();
-        if (auto encapsulatedSub = subPart.dynamicCast<MimeTreeParser::EncapsulatedRfc822MessagePart>()) {
-            findEncapsulated(encapsulatedSub);
-        }
-    }
-};
-
-PartModelPrivate::~PartModelPrivate()
-{
-}
-
 PartModel::PartModel(std::shared_ptr<MimeTreeParser::ObjectTreeParser> parser)
     : d(std::unique_ptr<PartModelPrivate>(new PartModelPrivate(this, parser)))
 {
@@ -76,6 +82,26 @@ PartModel::~PartModel()
 {
 }
 
+void PartModel::setShowHtml(bool html)
+{
+    if (d->showHtml != html) {
+        beginResetModel();
+        d->showHtml = html;
+        endResetModel();
+        emit showHtmlChanged();
+    }
+}
+
+bool PartModel::showHtml() const
+{
+    return d->showHtml;
+}
+
+bool PartModel::containsHtml() const
+{
+    return d->containsHtmlAndPlain;
+}
+
 QHash<int, QByteArray> PartModel::roleNames() const
 {
     QHash<int, QByteArray> roles;
@@ -85,6 +111,8 @@ QHash<int, QByteArray> PartModel::roleNames() const
     roles[IsEncryptedRole] = "encrypted";
     roles[IsSignedRole] = "signed";
     roles[SecurityLevelRole] = "securityLevel";
+    roles[EncryptionSecurityLevelRole] = "encryptionSecurityLevel";
+    roles[SignatureSecurityLevelRole] = "signatureSecurityLevel";
     roles[ErrorType] = "errorType";
     roles[ErrorString] = "errorString";
     roles[IsErrorRole] = "error";
@@ -206,6 +234,14 @@ QVariant PartModel::data(const QModelIndex &index, int role) const
                 if (dynamic_cast<MimeTreeParser::EncapsulatedRfc822MessagePart*>(messagePart)) {
                     return "encapsulated";
                 }
+                if (auto alternativePart = dynamic_cast<MimeTreeParser::AlternativeMessagePart*>(messagePart)) {
+                    if (alternativePart->availableModes().contains(MimeTreeParser::Util::MultipartIcal)) {
+                        return "ical";
+                    }
+                }
+                if (!d->showHtml && d->containsHtmlAndPlain) {
+                    return "plain";
+                }
                 //For simple html we don't need a browser
                 auto complexHtml = [&] {
                     if (messagePart->isHtml()) {
@@ -244,6 +280,14 @@ QVariant PartModel::data(const QModelIndex &index, int role) const
             case IsErrorRole:
                 return messagePart->error();
             case ContentRole: {
+                if (auto alternativePart = dynamic_cast<MimeTreeParser::AlternativeMessagePart*>(messagePart)) {
+                    if (alternativePart->availableModes().contains(MimeTreeParser::Util::MultipartIcal)) {
+                        return alternativePart->icalContent();
+                    }
+                }
+                if (!d->showHtml && d->containsHtmlAndPlain) {
+                    return HtmlUtils::linkify(Qt::convertFromPlainText(messagePart->isHtml() ? messagePart->plaintextContent() : messagePart->text()));
+                }
                 const auto text = messagePart->isHtml() ? messagePart->htmlContent() : messagePart->text();
                 if (messagePart->isHtml()) {
                     return addCss(d->mParser->resolveCidLinks(text));
@@ -267,12 +311,12 @@ QVariant PartModel::data(const QModelIndex &index, int role) const
 
                 if (messageIsSigned) {
                     auto sigInfo = std::unique_ptr<SignatureInfo>{signatureInfo(messagePart)};
-                    if (!sigInfo->signatureIsGood || sigInfo->keyRevoked) {
+                    if (!sigInfo->signatureIsGood) {
+                        if (sigInfo->keyMissing || sigInfo->keyExpired) {
+                            return "notsogood";
+                        }
                         return "bad";
                     }
-                    if (sigInfo->keyMissing || sigInfo->keyExpired) {
-                        return "notsogood";
-                    }
                 }
                 //All good
                 if (messageIsSigned || messageIsEncrypted) {
@@ -281,6 +325,38 @@ QVariant PartModel::data(const QModelIndex &index, int role) const
                 //No info
                 return "unknown";
             }
+            case EncryptionSecurityLevelRole: {
+                auto encryption = messagePart->encryptionState();
+                bool messageIsEncrypted = encryption == MimeTreeParser::KMMsgPartiallyEncrypted ||
+                                          encryption == MimeTreeParser::KMMsgFullyEncrypted;
+                if (messagePart->error()) {
+                    return "bad";
+                }
+                //All good
+                if (messageIsEncrypted) {
+                    return "good";
+                }
+                //No info
+                return "unknown";
+            }
+            case SignatureSecurityLevelRole: {
+                auto signature = messagePart->signatureState();
+                bool messageIsSigned = signature == MimeTreeParser::KMMsgPartiallySigned ||
+                                       signature == MimeTreeParser::KMMsgFullySigned;
+                if (messageIsSigned) {
+                    auto sigInfo = std::unique_ptr<SignatureInfo>{signatureInfo(messagePart)};
+                    if (!sigInfo->signatureIsGood) {
+                        if (sigInfo->keyMissing || sigInfo->keyExpired) {
+                            return "notsogood";
+                        }
+                        return "bad";
+                    }
+                    return "good";
+                }
+                //No info
+                return "unknown";
+
+            }
             case SignatureDetails:
                 return QVariant::fromValue(signatureInfo(messagePart));
             case EncryptionDetails:
