@@ -32,6 +32,8 @@ class PartModelPrivate;
 
 class PartModel : public QAbstractItemModel {
     Q_OBJECT
+    Q_PROPERTY(bool showHtml READ showHtml WRITE setShowHtml NOTIFY showHtmlChanged)
+    Q_PROPERTY(bool containsHtml READ containsHtml NOTIFY containsHtmlChanged)
 public:
     PartModel(std::shared_ptr<MimeTreeParser::ObjectTreeParser> parser);
     ~PartModel();
@@ -45,6 +47,8 @@ public:
         IsSignedRole,
         IsErrorRole,
         SecurityLevelRole,
+        EncryptionSecurityLevelRole,
+        SignatureSecurityLevelRole,
         SignatureDetails,
         EncryptionDetails,
         ErrorType,
@@ -60,6 +64,14 @@ public:
     int rowCount(const QModelIndex &parent = QModelIndex()) const Q_DECL_OVERRIDE;
     int columnCount(const QModelIndex &parent = QModelIndex()) const Q_DECL_OVERRIDE;
 
+    void setShowHtml(bool html);
+    bool showHtml() const;
+    bool containsHtml() const;
+
+signals:
+    void showHtmlChanged();
+    void containsHtmlChanged();
+
 private:
     std::unique_ptr<PartModelPrivate> d;
 };
