@@ -66,27 +66,27 @@ public:
                         }).exec();
                 }
             } else {
-                auto accountId = message["accountId"].value<QString>();
-                auto type = message["type"].value<QString>();
+                const auto accountId = message["accountId"].value<QString>();
+                const auto type = message["type"].value<QString>();
                 SyncScope scope;
                 if (!accountId.isEmpty()) {
                     //FIXME this should work with either string or bytearray, but is apparently very picky
                     scope.resourceFilter<SinkResource::Account>(accountId.toLatin1());
                 }
-                if (type == "contacts") {
-                    scope.setType<ApplicationDomain::Contact>();
-                } else if (type == "mail") {
-                    scope.setType<ApplicationDomain::Mail>();
-                } else if (type == "folder") {
-                    scope.setType<ApplicationDomain::Folder>();
-                } else {
-                    //Only synchronize folders by default for now
-                    scope.setType<ApplicationDomain::Folder>();
-                }
+                scope.setType(type.toUtf8());
                 SinkLog() << "Synchronizing... AccountId: " << accountId << " Type: " << scope.type();
                 Store::synchronize(scope).exec();
             }
         }
+        if (id == "abortSynchronization"/*Kube::Messages::abortSynchronization*/) {
+            const auto accountId = message["accountId"].value<QString>();
+            SyncScope scope;
+            if (!accountId.isEmpty()) {
+                //FIXME this should work with either string or bytearray, but is apparently very picky
+                scope.resourceFilter<SinkResource::Account>(accountId.toLatin1());
+            }
+            Store::abortSynchronization(scope).exec();
+        }
         if (id == "sendOutbox"/*Kube::Messages::synchronize*/) {
             Query query;
             query.containsFilter<SinkResource::Capabilities>(ResourceCapabilities::Mail::transport);
@@ -120,6 +120,12 @@ public:
                 Store::modify(*mail).exec();
             }
         }
+        if (id == "restoreFromTrash") {
+            if (auto mail = message["mail"].value<ApplicationDomain::Mail::Ptr>()) {
+                mail->setTrash(false);
+                Store::modify(*mail).exec();
+            }
+        }
         if (id == "moveToDrafts"/*Kube::Messages::synchronize*/) {
             if (auto mail = message["mail"].value<ApplicationDomain::Mail::Ptr>()) {
                 mail->setDraft(true);
@@ -189,8 +195,8 @@ public:
                         message["message"] = QObject::tr("Configuration error.");
                         break;
                     case Sink::ApplicationDomain::ConnectionLostError:
-                        message["message"] = QObject::tr("Connection lost.");
-                        break;
+                        //Ignore connection lost errors. We don't need them in the log view.
+                        return;
                     case Sink::ApplicationDomain::MissingCredentialsError:
                         message["message"] = QObject::tr("No credentials available.");
                         break;
@@ -203,11 +209,30 @@ public:
                 if (notification.code == Sink::ApplicationDomain::TransmissionSuccess) {
                     message["type"] = "info";
                     message["message"] = QObject::tr("A message has been sent.");
+                    message["subtype"] = "messageSent";
+
+                    QVariantList entities;
+                    for(const auto &entity : notification.entities) {
+                        entities << entity;
+                    }
+                    message["entities"] = entities;
+
+                    message["resource"] = QString{notification.resource};
                 } else if (notification.code == Sink::ApplicationDomain::NewContentAvailable) {
                     message["type"] = "info";
                     if (!notification.entities.isEmpty()) {
                         message["folderId"] = notification.entities.first();
                     }
+                } else if (notification.code == Sink::ApplicationDomain::SyncInProgress) {
+                    message["type"] = "progress";
+                    message["progress"] = 0;
+                    message["total"] = 1;
+                    if (!notification.entities.isEmpty()) {
+                        message["folderId"] = notification.entities.first();
+                    }
+                    message["resourceId"] = notification.resource;
+                    Fabric::Fabric{}.postMessage("progressNotification", message);
+                    return;
                 } else {
                     return;
                 }
